@@ -10,8 +10,8 @@
  * @brief standard mathematical functions for xexpressions
  */
 
-#ifndef XBUILDER_HPP
-#define XBUILDER_HPP
+#ifndef XTENSOR_BUILDER_HPP
+#define XTENSOR_BUILDER_HPP
 
 #include <array>
 #include <cmath>
@@ -29,6 +29,7 @@
 #include "xbroadcast.hpp"
 #include "xfunction.hpp"
 #include "xgenerator.hpp"
+#include "xoperation.hpp"
 
 namespace xt
 {
@@ -191,7 +192,7 @@ namespace xt
             inline T operator()(const It& /*begin*/, const It& end) const
             {
                 using lvalue_type = typename std::iterator_traits<It>::value_type;
-                return *(end - 1) == *(end - 2) + lvalue_type(m_k) ? T(1) : T(0);
+                return *(end - 1) == *(end - 2) + static_cast<lvalue_type>(static_cast<unsigned int>(m_k)) ? T(1) : T(0);
             }
 
         private:
@@ -270,8 +271,9 @@ namespace xt
     template <class T>
     inline auto linspace(T start, T stop, std::size_t num_samples = 50, bool endpoint = true) noexcept
     {
-        T step = (stop - start) / T(num_samples - (endpoint ? 1 : 0));
-        return detail::make_xgenerator(detail::arange_impl<T>(start, stop, step), {num_samples});
+        using fp_type = std::common_type_t<T, double>;
+        fp_type step = fp_type(stop - start) / fp_type(num_samples - (endpoint ? 1 : 0));
+        return cast<T>(detail::make_xgenerator(detail::arange_impl<fp_type>(fp_type(start), fp_type(stop), step), {num_samples}));
     }
 
     /**
@@ -287,7 +289,7 @@ namespace xt
     template <class T>
     inline auto logspace(T start, T stop, std::size_t num_samples, T base = 10, bool endpoint = true) noexcept
     {
-        return pow(std::forward<T>(base), linspace(start, stop, num_samples, endpoint));
+        return cast<T>(pow(std::move(base), linspace(start, stop, num_samples, endpoint)));
     }
 
     namespace detail
@@ -323,8 +325,7 @@ namespace xt
 
             inline value_type access_impl(xindex idx) const
             {
-                auto match = [this, &idx](auto& arr)
-                {
+                auto match = [this, &idx](auto& arr) {
                     if (idx[this->m_axis] >= arr.shape()[this->m_axis])
                     {
                         idx[this->m_axis] -= arr.shape()[this->m_axis];
@@ -333,8 +334,7 @@ namespace xt
                     return true;
                 };
 
-                auto get = [&idx](auto& arr)
-                {
+                auto get = [&idx](auto& arr) {
                     return arr[idx];
                 };
 
@@ -384,8 +384,7 @@ namespace xt
 
             inline value_type access_impl(xindex idx) const
             {
-                auto get_item = [&idx](auto& arr)
-                {
+                auto get_item = [&idx](auto& arr) {
                     return arr[idx];
                 };
                 size_type i = idx[m_axis];
@@ -523,7 +522,7 @@ namespace xt
         inline auto meshgrid_impl(std::index_sequence<I...>, E&&... e) noexcept
         {
 #if defined X_OLD_CLANG || defined _MSC_VER
-            const std::array<std::size_t, sizeof...(E)> shape { e.shape()[0]... };
+            const std::array<std::size_t, sizeof...(E)> shape = {e.shape()[0]...};
             return std::make_tuple(
                 detail::make_xgenerator(
                     detail::repeat_impl<xclosure_t<E>>(std::forward<E>(e), I),
@@ -534,7 +533,7 @@ namespace xt
             return std::make_tuple(
                 detail::make_xgenerator(
                     detail::repeat_impl<xclosure_t<E>>(std::forward<E>(e), I),
-                    { e.shape()[0]... }
+                    {e.shape()[0]...}
                 )...
             );
 #endif
