@@ -9,7 +9,7 @@
 /**
  * @brief standard mathematical functions for xexpressions
  */
-
+\
 #ifndef XTENSOR_MATH_HPP
 #define XTENSOR_MATH_HPP
 
@@ -19,12 +19,13 @@
 
 #include "xoperation.hpp"
 #include "xreducer.hpp"
+#include "xaccumulator.hpp"
 
 #include "xtl/xcomplex.hpp"
 
 namespace xt
 {
-    template <class T>
+    template <class T = double>
     struct numeric_constants
     {
         static constexpr T PI = 3.141592653589793238463;
@@ -45,92 +46,93 @@ namespace xt
      * Helpers *
      ***********/
 
-#define UNSIGNED_ABS_FUNCTOR(T)                                                 \
-    template <>                                                                 \
-    struct abs_fun<T>                                                           \
-    {                                                                           \
-        using return_type = xt::detail::functor_return_type<T, T>;              \
-        using argument_type = T;                                                \
-        using result_type = T;                                                  \
-        using simd_value_type = xsimd::simd_type<T>;                            \
-        using simd_result_type = typename return_type::simd_type;               \
-        constexpr result_type operator()(const T& arg) const                    \
-        {                                                                       \
-            return arg;                                                         \
-        }                                                                       \
-        constexpr simd_result_type simd_apply(const simd_value_type& arg) const \
-        {                                                                       \
-            return arg;                                                         \
-        }                                                                       \
-        template <class U>                                                      \
-        struct rebind                                                           \
-        {                                                                       \
-            using type = abs_fun<U>;                                            \
-        };                                                                      \
-    }
+#define UNSIGNED_ABS_FUNC(T)                                                     \
+constexpr inline T abs(const T& x)                                               \
+{                                                                                \
+    return x;                                                                    \
+}                                                                                \
 
-#define UNARY_MATH_FUNCTOR_IMPL(NAME, R)                                        \
-    template <class T>                                                          \
-    struct NAME##_fun                                                           \
-    {                                                                           \
-        static auto exec(const T& arg)                                          \
-        {                                                                       \
-            using std::NAME;                                                    \
-            return NAME(arg);                                                   \
-        }                                                                       \
-        using return_type = xt::detail::functor_return_type<T, R>;              \
-        using argument_type = T;                                                \
-        using result_type = decltype(exec(*(T*)0));                             \
-        using simd_value_type = xsimd::simd_type<T>;                            \
-        using simd_result_type = typename return_type::simd_type;               \
-        constexpr result_type operator()(const T& arg) const                    \
-        {                                                                       \
-            using std::NAME;                                                    \
-            return NAME(arg);                                                   \
-        }                                                                       \
-        constexpr simd_result_type simd_apply(const simd_value_type& arg) const \
-        {                                                                       \
-            using std::NAME;                                                    \
-            return NAME(arg);                                                   \
-        }                                                                       \
-        template <class U>                                                      \
-        struct rebind                                                           \
-        {                                                                       \
-            using type = NAME##_fun<U>;                                         \
-        };                                                                      \
+#define INT_SPECIALIZATION_IMPL(FUNC_NAME, RETURN_VAL, T)                        \
+constexpr inline bool FUNC_NAME(const T& /*x*/) noexcept                         \
+{                                                                                \
+    return RETURN_VAL;                                                           \
+}                                                                                \
+
+#define INT_SPECIALIZATION(FUNC_NAME, RETURN_VAL)                                \
+INT_SPECIALIZATION_IMPL(FUNC_NAME, RETURN_VAL, char);                            \
+INT_SPECIALIZATION_IMPL(FUNC_NAME, RETURN_VAL, short);                           \
+INT_SPECIALIZATION_IMPL(FUNC_NAME, RETURN_VAL, int);                             \
+INT_SPECIALIZATION_IMPL(FUNC_NAME, RETURN_VAL, long);                            \
+INT_SPECIALIZATION_IMPL(FUNC_NAME, RETURN_VAL, long long);                       \
+INT_SPECIALIZATION_IMPL(FUNC_NAME, RETURN_VAL, unsigned char);                   \
+INT_SPECIALIZATION_IMPL(FUNC_NAME, RETURN_VAL, unsigned short);                  \
+INT_SPECIALIZATION_IMPL(FUNC_NAME, RETURN_VAL, unsigned int);                    \
+INT_SPECIALIZATION_IMPL(FUNC_NAME, RETURN_VAL, unsigned long);                   \
+INT_SPECIALIZATION_IMPL(FUNC_NAME, RETURN_VAL, unsigned long long);              \
+
+
+#define UNARY_MATH_FUNCTOR_IMPL(NAME, R)                                         \
+    template <class T>                                                           \
+    struct NAME##_fun                                                            \
+    {                                                                            \
+        static auto exec(const T& arg)                                           \
+        {                                                                        \
+            using math::NAME;                                                    \
+            return NAME(arg);                                                    \
+        }                                                                        \
+        using return_type = xt::detail::functor_return_type<T, R>;               \
+        using argument_type = T;                                                 \
+        using result_type = decltype(exec(*(T*)0));                              \
+        using simd_value_type = xsimd::simd_type<T>;                             \
+        using simd_result_type = typename return_type::simd_type;                \
+        constexpr result_type operator()(const T& arg) const                     \
+        {                                                                        \
+            using math::NAME;                                                    \
+            return NAME(arg);                                                    \
+        }                                                                        \
+        constexpr simd_result_type simd_apply(const simd_value_type& arg) const  \
+        {                                                                        \
+            using math::NAME;                                                    \
+            return NAME(arg);                                                    \
+        }                                                                        \
+        template <class U>                                                       \
+        struct rebind                                                            \
+        {                                                                        \
+            using type = NAME##_fun<U>;                                          \
+        };                                                                       \
     }
 
 #define UNARY_MATH_FUNCTOR(NAME) UNARY_MATH_FUNCTOR_IMPL(NAME, T)
 #define UNARY_BOOL_FUNCTOR(NAME) UNARY_MATH_FUNCTOR_IMPL(NAME, bool)
 
-#define UNARY_MATH_FUNCTOR_COMPLEX_REDUCING(NAME)                               \
-    template <class T>                                                          \
-    struct NAME##_fun                                                           \
-    {                                                                           \
-        static auto exec(const T& arg)                                          \
-        {                                                                       \
-            using std::NAME;                                                    \
-            return NAME(arg);                                                   \
-        }                                                                       \
-        using argument_type = T;                                                \
-        using result_type = decltype(exec(*(T*)0));                             \
-        using simd_value_type = argument_type;                                  \
-        using simd_result_type = result_type;                                   \
-        constexpr result_type operator()(const T& arg) const                    \
-        {                                                                       \
-            using std::NAME;                                                    \
-            return NAME(arg);                                                   \
-        }                                                                       \
-        constexpr simd_result_type simd_apply(const simd_value_type& arg) const \
-        {                                                                       \
-            using std::NAME;                                                    \
-            return NAME(arg);                                                   \
-        }                                                                       \
-        template <class U>                                                      \
-        struct rebind                                                           \
-        {                                                                       \
-            using type = NAME##_fun<U>;                                         \
-        };                                                                      \
+#define UNARY_MATH_FUNCTOR_COMPLEX_REDUCING(NAME)                                \
+    template <class T>                                                           \
+    struct NAME##_fun                                                            \
+    {                                                                            \
+        static auto exec(const T& arg)                                           \
+        {                                                                        \
+            using math::NAME;                                                    \
+            return NAME(arg);                                                    \
+        }                                                                        \
+        using argument_type = T;                                                 \
+        using result_type = decltype(exec(*(T*)0));                              \
+        using simd_value_type = argument_type;                                   \
+        using simd_result_type = result_type;                                    \
+        constexpr result_type operator()(const T& arg) const                     \
+        {                                                                        \
+            using math::NAME;                                                    \
+            return NAME(arg);                                                    \
+        }                                                                        \
+        constexpr simd_result_type simd_apply(const simd_value_type& arg) const  \
+        {                                                                        \
+            using math::NAME;                                                    \
+            return NAME(arg);                                                    \
+        }                                                                        \
+        template <class U>                                                       \
+        struct rebind                                                            \
+        {                                                                        \
+            using type = NAME##_fun<U>;                                          \
+        };                                                                       \
     }
 
 #define BINARY_MATH_FUNCTOR_IMPL(NAME, R)                                        \
@@ -139,7 +141,7 @@ namespace xt
     {                                                                            \
         static auto exec(const T& arg1, const T& arg2)                           \
         {                                                                        \
-            using std::NAME;                                                     \
+            using math::NAME;                                                    \
             return NAME(arg1, arg2);                                             \
         }                                                                        \
         using return_type = xt::detail::functor_return_type<T, R>;               \
@@ -150,13 +152,13 @@ namespace xt
         using simd_result_type = typename return_type::simd_type;                \
         constexpr result_type operator()(const T& arg1, const T& arg2) const     \
         {                                                                        \
-            using std::NAME;                                                     \
+            using math::NAME;                                                    \
             return NAME(arg1, arg2);                                             \
         }                                                                        \
         constexpr simd_result_type simd_apply(const simd_value_type& arg1,       \
                                               const simd_value_type& arg2) const \
         {                                                                        \
-            using std::NAME;                                                     \
+            using math::NAME;                                                    \
             return NAME(arg1, arg2);                                             \
         }                                                                        \
         template <class U>                                                       \
@@ -175,7 +177,7 @@ namespace xt
     {                                                                            \
         static auto exec(const T& arg1, const T& arg2, const T& arg3)            \
         {                                                                        \
-            using std::NAME;                                                     \
+            using math::NAME;                                                    \
             return NAME(arg1, arg2, arg3);                                       \
         }                                                                        \
         using return_type = xt::detail::functor_return_type<T, R>;               \
@@ -189,14 +191,14 @@ namespace xt
                                          const T& arg2,                          \
                                          const T& arg3) const                    \
         {                                                                        \
-            using std::NAME;                                                     \
+            using math::NAME;                                                    \
             return NAME(arg1, arg2, arg3);                                       \
         }                                                                        \
         constexpr simd_result_type simd_apply(const simd_value_type& arg1,       \
                                               const simd_value_type& arg2,       \
                                               const simd_value_type& arg3) const \
         {                                                                        \
-            using std::NAME;                                                     \
+            using math::NAME;                                                    \
             return NAME(arg1, arg2, arg3);                                       \
         }                                                                        \
         template <class U>                                                       \
@@ -211,12 +213,106 @@ namespace xt
 
     namespace math
     {
-        UNARY_MATH_FUNCTOR_COMPLEX_REDUCING(abs);
+        using std::abs;
+        using std::fabs;
+
+        using std::cos;
+        using std::sin;
+        using std::tan;
+        using std::acos;
+        using std::asin;
+        using std::atan;
+
+        using std::cosh;
+        using std::sinh;
+        using std::tanh;
+        using std::acosh;
+        using std::asinh;
+        using std::atanh;
+
+        using std::sqrt;
+        using std::cbrt;
+
+        using std::exp;
+        using std::exp2;
+        using std::expm1;
+        using std::log;
+        using std::log2;
+        using std::log10;
+        using std::log1p;
+        using std::logb;
+        using std::ilogb;
+
+        using std::floor;
+        using std::ceil;
+        using std::trunc;
+        using std::round;
+        using std::lround;
+        using std::llround;
+        using std::rint;
+        using std::nearbyint;
+        using std::remainder;
+
+        using std::erf;
+        using std::erfc;
+        using std::erfc;
+        using std::tgamma;
+        using std::lgamma;
+
+        using std::conj;
+        using std::real;
+        using std::imag;
+        using std::arg;
+
+        using std::atan2;
+        using std::copysign;
+        using std::fdim;
+        using std::fmax;
+        using std::fmin;
+        using std::fmod;
+        using std::hypot;
+        using std::pow;
+
+        using std::fma;
+
+        using std::isnan;
+        using std::isinf;
+        using std::isfinite;
+        using std::fpclassify;
+
+        // Overload isinf, isnan and isfinite for complex datatypes,
+        // following the Python specification:
+        template <class T>
+        inline bool isinf(const std::complex<T>& c)
+        {
+            return std::isinf(std::real(c)) || std::isinf(std::imag(c));
+        }
+
+        template <class T>
+        inline bool isnan(const std::complex<T>& c)
+        {
+            return std::isnan(std::real(c)) || std::isnan(std::imag(c));
+        }
+
+        template <class T>
+        inline bool isfinite(const std::complex<T>& c)
+        {
+            return !isinf(c) && !isnan(c);
+        }
+
         // The following specializations are needed to avoid 'ambiguous overload' errors,
         // whereas 'unsigned char' and 'unsigned short' are automatically converted to 'int'.
-        UNSIGNED_ABS_FUNCTOR(unsigned int);
-        UNSIGNED_ABS_FUNCTOR(unsigned long);
-        UNSIGNED_ABS_FUNCTOR(unsigned long long);
+        UNSIGNED_ABS_FUNC(unsigned int);
+        UNSIGNED_ABS_FUNC(unsigned long);
+        UNSIGNED_ABS_FUNC(unsigned long long);
+
+#ifdef _WIN32
+        INT_SPECIALIZATION(isinf, false);
+        INT_SPECIALIZATION(isnan, false);
+        INT_SPECIALIZATION(isfinite, true);
+#endif
+
+        UNARY_MATH_FUNCTOR_COMPLEX_REDUCING(abs);
 
         UNARY_MATH_FUNCTOR(fabs);
         BINARY_MATH_FUNCTOR(fmod);
@@ -275,6 +371,46 @@ namespace xt
 #undef TERNARY_MATH_FUNCTOR_IMPL
 #undef UNARY_MATH_FUNCTOR_COMPLEX_REDUCING
 #undef UNSIGNED_ABS_FUNCTOR
+#undef UNSIGNED_ABS_FUNC
+
+#define REDUCER_FUNCTION(NAME, FUNCTOR, RESULT_TYPE)                                                              \
+    template <class E, class X, class ES = DEFAULT_STRATEGY_REDUCERS,                                             \
+              class = std::enable_if_t<!std::is_base_of<evaluation_strategy::base, std::decay_t<X>>::value, int>> \
+    inline auto NAME(E&& e, X&& axes, ES es = ES()) noexcept                                                      \
+    {                                                                                                             \
+        using result_type = RESULT_TYPE;                                                                          \
+        using functor_type = FUNCTOR<result_type>;                                                                \
+        return reduce(make_xreducer_functor(functor_type()), std::forward<E>(e),                                  \
+                      std::forward<X>(axes), es);                                                                 \
+    }                                                                                                             \
+                                                                                                                  \
+    template <class E, class ES = DEFAULT_STRATEGY_REDUCERS,                                                      \
+              class = std::enable_if_t<std::is_base_of<evaluation_strategy::base, ES>::value, int>>               \
+    inline auto NAME(E&& e, ES es = ES()) noexcept                                                                \
+    {                                                                                                             \
+        using result_type = RESULT_TYPE;                                                                          \
+        using functor_type = FUNCTOR<result_type>;                                                                \
+        return reduce(make_xreducer_functor(functor_type()), std::forward<E>(e), es);                             \
+    }                                                                                                             \
+
+#define OLD_CLANG_REDUCER(NAME, FUNCTOR, RESULT_TYPE)                                                             \
+    template <class E, class I, class ES = DEFAULT_STRATEGY_REDUCERS>                                             \
+        inline auto NAME(E&& e, std::initializer_list<I> axes, ES es = ES()) noexcept                             \
+        {                                                                                                         \
+            using result_type = RESULT_TYPE;                                                                      \
+            using functor_type = FUNCTOR<result_type>;                                                            \
+            return reduce(make_xreducer_functor(functor_type()), std::forward<E>(e), axes);                       \
+        }                                                                                                         \
+
+#define MODERN_CLANG_REDUCER(NAME, FUNCTOR, RESULT_TYPE)                                                          \
+    template <class E, class I, std::size_t N, class ES = DEFAULT_STRATEGY_REDUCERS>                              \
+    inline auto NAME(E&& e, const I (&axes)[N], ES es = ES()) noexcept                                            \
+    {                                                                                                             \
+        using result_type = RESULT_TYPE;                                                                          \
+        using functor_type = FUNCTOR<result_type>;                                                                \
+        return reduce(make_xreducer_functor(functor_type()), std::forward<E>(e), axes, es);                       \
+    }                                                                                                             \
+
 
     /*******************
      * basic functions *
@@ -526,40 +662,14 @@ namespace xt
      * \em axes.
      * @param e an \ref xexpression
      * @param axes the axes along which the maximum is found (optional)
+     * @param es evaluation strategy of the reducer
      * @return an \ref xreducer
      */
-    template <class E, class X>
-    inline auto amax(E&& e, X&& axes) noexcept
-    {
-        using result_type = typename std::decay_t<E>::value_type;
-        using functor_type = math::maximum<result_type>;
-        return reduce(make_xreducer_functor(functor_type()), std::forward<E>(e), std::forward<X>(axes));
-    }
-
-    template <class E>
-    inline auto amax(E&& e) noexcept
-    {
-        using result_type = typename std::decay_t<E>::value_type;
-        using functor_type = math::maximum<result_type>;
-        return reduce(make_xreducer_functor(functor_type()), std::forward<E>(e));
-    }
-
+    REDUCER_FUNCTION(amax, math::maximum, typename std::decay_t<E>::value_type);
 #ifdef X_OLD_CLANG
-    template <class E, class I>
-    inline auto amax(E&& e, std::initializer_list<I> axes) noexcept
-    {
-        using result_type = typename std::decay_t<E>::value_type;
-        using functor_type = math::maximum<result_type>;
-        return reduce(make_xreducer_functor(functor_type()), std::forward<E>(e), axes);
-    }
+    OLD_CLANG_REDUCER(amax, math::maximum, typename std::decay_t<E>::value_type);
 #else
-    template <class E, class I, std::size_t N>
-    inline auto amax(E&& e, const I (&axes)[N]) noexcept
-    {
-        using result_type = typename std::decay_t<E>::value_type;
-        using functor_type = math::maximum<result_type>;
-        return reduce(make_xreducer_functor(functor_type()), std::forward<E>(e), axes);
-    }
+    MODERN_CLANG_REDUCER(amax, math::maximum, typename std::decay_t<E>::value_type);
 #endif
 
     /**
@@ -570,40 +680,14 @@ namespace xt
      * \em axes.
      * @param e an \ref xexpression
      * @param axes the axes along which the minimum is found (optional)
+     * @param es evaluation strategy of the reducer
      * @return an \ref xreducer
      */
-    template <class E, class X>
-    inline auto amin(E&& e, X&& axes) noexcept
-    {
-        using result_type = typename std::decay_t<E>::value_type;
-        using functor_type = math::minimum<result_type>;
-        return reduce(make_xreducer_functor(functor_type()), std::forward<E>(e), std::forward<X>(axes));
-    }
-
-    template <class E>
-    inline auto amin(E&& e) noexcept
-    {
-        using result_type = typename std::decay_t<E>::value_type;
-        using functor_type = math::minimum<result_type>;
-        return reduce(make_xreducer_functor(functor_type()), std::forward<E>(e));
-    }
-
+    REDUCER_FUNCTION(amin, math::minimum, typename std::decay_t<E>::value_type);
 #ifdef X_OLD_CLANG
-    template <class E, class I>
-    inline auto amin(E&& e, std::initializer_list<I> axes) noexcept
-    {
-        using result_type = typename std::decay_t<E>::value_type;
-        using functor_type = math::minimum<result_type>;
-        return reduce(make_xreducer_functor(functor_type()), std::forward<E>(e), axes);
-    }
+    OLD_CLANG_REDUCER(amin, math::minimum, typename std::decay_t<E>::value_type);
 #else
-    template <class E, class I, std::size_t N>
-    inline auto amin(E&& e, const I (&axes)[N]) noexcept
-    {
-        using result_type = typename std::decay_t<E>::value_type;
-        using functor_type = math::minimum<result_type>;
-        return reduce(make_xreducer_functor(functor_type()), std::forward<E>(e), axes);
-    }
+    MODERN_CLANG_REDUCER(amin, math::minimum, typename std::decay_t<E>::value_type);
 #endif
 
     /**
@@ -1375,18 +1459,18 @@ namespace xt
 
             bool operator()(const T& a, const T& b) const
             {
-                using std::abs;
-
-                if (std::isnan(a) && std::isnan(b))
+                using internal_type = promote_type_t<T, double>;
+                if(math::isnan(a) && math::isnan(b))
                 {
                     return m_equal_nan;
                 }
-                if (std::isinf(a) && std::isinf(b))
+                if(math::isinf(a) && math::isinf(b))
                 {
-                    return std::signbit(a) == std::signbit(b);
+                    // check for both infinity signs equal
+                    return a == b;
                 }
-                auto d = abs(a - b);
-                return d <= m_atol || d <= m_rtol * std::max(abs(a), abs(b));
+                auto d = math::abs((internal_type) a - (internal_type) b);
+                return d <= m_atol || d <= m_rtol * (double) std::max(math::abs(a), math::abs(b));
             }
 
             template <class U>
@@ -1447,6 +1531,7 @@ namespace xt
      * Reducing functions *
      **********************/
 
+
     /**
      * @defgroup  red_functions reducing functions
      */
@@ -1459,40 +1544,14 @@ namespace xt
      * \em axes.
      * @param e an \ref xexpression
      * @param axes the axes along which the sum is performed (optional)
+     * @param es evaluation strategy of the reducer
      * @return an \ref xreducer
      */
-    template <class E, class X>
-    inline auto sum(E&& e, X&& axes) noexcept
-    {
-        using result_type = big_promote_type_t<typename std::decay_t<E>::value_type>;
-        using functor_type = std::plus<result_type>;
-        return reduce(make_xreducer_functor(functor_type()), std::forward<E>(e), std::forward<X>(axes));
-    }
-
-    template <class E>
-    inline auto sum(E&& e) noexcept
-    {
-        using result_type = big_promote_type_t<typename std::decay_t<E>::value_type>;
-        using functor_type = std::plus<result_type>;
-        return reduce(make_xreducer_functor(functor_type()), std::forward<E>(e));
-    }
-
+    REDUCER_FUNCTION(sum, std::plus, big_promote_type_t<typename std::decay_t<E>::value_type>);
 #ifdef X_OLD_CLANG
-    template <class E, class I>
-    inline auto sum(E&& e, std::initializer_list<I> axes) noexcept
-    {
-        using result_type = big_promote_type_t<typename std::decay_t<E>::value_type>;
-        using functor_type = std::plus<result_type>;
-        return reduce(make_xreducer_functor(functor_type()), std::forward<E>(e), axes);
-    }
+    OLD_CLANG_REDUCER(sum, std::plus, big_promote_type_t<typename std::decay_t<E>::value_type>);
 #else
-    template <class E, class I, std::size_t N>
-    inline auto sum(E&& e, const I (&axes)[N]) noexcept
-    {
-        using result_type = big_promote_type_t<typename std::decay_t<E>::value_type>;
-        using functor_type = std::plus<result_type>;
-        return reduce(make_xreducer_functor(functor_type()), std::forward<E>(e), axes);
-    }
+    MODERN_CLANG_REDUCER(sum, std::plus, big_promote_type_t<typename std::decay_t<E>::value_type>);
 #endif
 
     /**
@@ -1503,40 +1562,14 @@ namespace xt
      * \em axes.
      * @param e an \ref xexpression
      * @param axes the axes along which the product is computed (optional)
+     * @param es evaluation strategy of the reducer
      * @return an \ref xreducer
      */
-    template <class E, class X>
-    inline auto prod(E&& e, X&& axes) noexcept
-    {
-        using result_type = big_promote_type_t<typename std::decay_t<E>::value_type>;
-        using functor_type = std::multiplies<result_type>;
-        return reduce(make_xreducer_functor(functor_type()), std::forward<E>(e), std::forward<X>(axes));
-    }
-
-    template <class E>
-    inline auto prod(E&& e) noexcept
-    {
-        using result_type = big_promote_type_t<typename std::decay_t<E>::value_type>;
-        using functor_type = std::multiplies<result_type>;
-        return reduce(make_xreducer_functor(functor_type()), std::forward<E>(e));
-    }
-
+    REDUCER_FUNCTION(prod, std::multiplies, big_promote_type_t<typename std::decay_t<E>::value_type>);
 #ifdef X_OLD_CLANG
-    template <class E, class I>
-    inline auto prod(E&& e, std::initializer_list<I> axes) noexcept
-    {
-        using result_type = big_promote_type_t<typename std::decay_t<E>::value_type>;
-        using functor_type = std::multiplies<result_type>;
-        return reduce(make_xreducer_functor(functor_type()), std::forward<E>(e), axes);
-    }
+    OLD_CLANG_REDUCER(prod, std::multiplies, big_promote_type_t<typename std::decay_t<E>::value_type>);
 #else
-    template <class E, class I, std::size_t N>
-    inline auto prod(E&& e, const I (&axes)[N]) noexcept
-    {
-        using result_type = big_promote_type_t<typename std::decay_t<E>::value_type>;
-        using functor_type = std::multiplies<result_type>;
-        return reduce(make_xreducer_functor(functor_type()), std::forward<E>(e), axes);
-    }
+    MODERN_CLANG_REDUCER(prod, std::multiplies, big_promote_type_t<typename std::decay_t<E>::value_type>);
 #endif
 
     /**
@@ -1581,6 +1614,58 @@ namespace xt
         return std::move(s) / static_cast<double>(size / s.size());
     }
 #endif
+
+    /**
+     * @defgroup acc_functions accumulating functions
+     */
+
+    /**
+     * @ingroup acc_functions
+     * @brief Cumulative sum.
+     *
+     * Returns the accumulated sum for the elements over given
+     * \em axis (or flattened).
+     * @param e an \ref xexpression
+     * @param axis the axes along which the cumulative sum is computed (optional)
+     * @return an \ref xarray<T>
+     */
+    template <class E>
+    inline auto cumsum(E&& e, std::size_t axis) noexcept
+    {
+        using result_type = big_promote_type_t<typename std::decay_t<E>::value_type>;
+        return accumulate(std::plus<result_type>(), std::forward<E>(e), axis);
+    }
+
+    template <class E>
+    inline auto cumsum(E&& e) noexcept
+    {
+        using result_type = big_promote_type_t<typename std::decay_t<E>::value_type>;
+        return accumulate(std::plus<result_type>(), std::forward<E>(e));
+    }
+
+    /**
+     * @ingroup acc_functions
+     * @brief Cumulative product.
+     *
+     * Returns the accumulated product for the elements over given
+     * \em axis (or flattened).
+     * @param e an \ref xexpression
+     * @param axis the axes along which the cumulative product is computed (optional)
+     * @return an \ref xarray<T>
+     */
+    template <class E>
+    inline auto cumprod(E&& e, std::size_t axis) noexcept
+    {
+        using result_type = big_promote_type_t<typename std::decay_t<E>::value_type>;
+        return accumulate(std::multiplies<result_type>(), std::forward<E>(e), axis);
+    }
+
+    template <class E>
+    inline auto cumprod(E&& e) noexcept
+    {
+        using result_type = big_promote_type_t<typename std::decay_t<E>::value_type>;
+        return accumulate(std::multiplies<result_type>(), std::forward<E>(e));
+    }
 }
 
 #endif
