@@ -50,6 +50,9 @@ namespace xt
         template <class E>
         disable_xexpression<E, derived_type&> operator/=(const E&);
 
+        template <class E>
+        disable_xexpression<E, derived_type&> operator%=(const E&);
+        
         template <class E>
         disable_xexpression<E, derived_type&> operator&=(const E&);
 
@@ -71,6 +74,9 @@ namespace xt
         template <class E>
         derived_type& operator/=(const xexpression<E>&);
 
+        template <class E>
+        derived_type& operator%=(const xexpression<E>&);
+
         template <class E>
         derived_type& operator&=(const xexpression<E>&);
 
@@ -94,7 +100,10 @@ namespace xt
 
         template <class E>
         derived_type& divides_assign(const xexpression<E>&);
-
+        
+        template <class E>
+        derived_type& modulus_assign(const xexpression<E>&);
+        
     protected:
 
         xsemantic_base() = default;
@@ -257,6 +266,18 @@ namespace xt
         return this->derived_cast().scalar_computed_assign(e, std::divides<>());
     }
 
+    /**
+     * Computes the remainder of \c *this after division by the scalar \c e.
+     * @param e the scalar involved in the operation.
+     * @return a reference to \c *this.
+     */
+    template <class D>
+    template <class E>
+    inline auto xsemantic_base<D>::operator%=(const E& e) -> disable_xexpression<E, derived_type&>
+    {
+        return this->derived_cast().scalar_computed_assign(e, std::modulus<>());
+    }
+    
     /**
      * Computes the bitwise and of \c *this and the scalar \c e and assigns it to \c *this.
      * @param e the scalar involved in the operation.
@@ -341,6 +362,18 @@ namespace xt
         return operator=(this->derived_cast() / e.derived_cast());
     }
 
+    /**
+     * Computes the remainder of \c *this after division by the xexpression \c e.
+     * @param e the xexpression involved in the operation.
+     * @return a reference to \c *this.
+     */
+    template <class D>
+    template <class E>
+    inline auto xsemantic_base<D>::operator%=(const xexpression<E>& e) -> derived_type&
+    {
+        return operator=(this->derived_cast() % e.derived_cast());
+    }
+    
     /**
      * Computes the bitwise and of \c *this and the xexpression \c e and assigns it to \c *this.
      * @param e the xexpression involved in the operation.
@@ -446,6 +479,19 @@ namespace xt
         return this->derived_cast().computed_assign(this->derived_cast() / e.derived_cast());
     }
 
+    /**
+     * Computes the remainder of \c *this after division by the xexpression \c e.
+     * Ensures no temporary will be used to perform the assignment.
+     * @param e the xexpression involved in the operation.
+     * @return a reference to \c *this.
+     */
+    template <class D>
+    template <class E>
+    inline auto xsemantic_base<D>::modulus_assign(const xexpression<E>& e) -> derived_type&
+    {
+        return this->derived_cast().computed_assign(this->derived_cast() % e.derived_cast());
+    }
+    
     template <class D>
     template <class E>
     inline auto xsemantic_base<D>::operator=(const xexpression<E>& e) -> derived_type&
