@@ -10,7 +10,7 @@
 #define XTENSOR_CONFIG_HPP
 
 #define XTENSOR_VERSION_MAJOR 0
-#define XTENSOR_VERSION_MINOR 13
+#define XTENSOR_VERSION_MINOR 14
 #define XTENSOR_VERSION_PATCH 1
 
 // DETECT 3.6 <= clang < 3.8 for compiler bug workaround.
@@ -31,6 +31,17 @@
     std::vector<typename DEFAULT_DATA_CONTAINER(T, EA)::size_type, SA>
 #endif
 
+#ifndef DEFAULT_ALLOCATOR
+#ifdef XTENSOR_USE_XSIMD
+#include "xsimd/xsimd.hpp"
+#define DEFAULT_ALLOCATOR(T) \
+    xsimd::aligned_allocator<T, XSIMD_DEFAULT_ALIGNMENT>
+#else
+#define DEFAULT_ALLOCATOR(T) \
+    std::allocator<T>
+#endif
+#endif
+
 #ifndef DEFAULT_LAYOUT
 #define DEFAULT_LAYOUT layout_type::row_major
 #endif
