@@ -47,7 +47,7 @@ namespace xt
      * \code{.cpp}
      * xt::xarray_container<std::vector<double>, std::vector<std::size_t>> a = ...
      * \endcode
-     * 
+     *
      * @tparam T The value type of the elements.
      * @tparam L The layout_type of the xarray_container (default: row_major).
      * @tparam A The allocator of the container holding the elements.
@@ -55,7 +55,7 @@ namespace xt
      */
     template <class T,
               layout_type L = DEFAULT_LAYOUT,
-              class A = std::allocator<T>,
+              class A = DEFAULT_ALLOCATOR(T),
               class SA = std::allocator<typename std::vector<T, A>::size_type>>
     using xarray = xarray_container<DEFAULT_DATA_CONTAINER(T, A), L, DEFAULT_SHAPE_CONTAINER(T, A, SA)>;
 
@@ -77,7 +77,7 @@ namespace xt
      */
     template <class T,
               layout_type L = DEFAULT_LAYOUT,
-              class A = std::allocator<T>,
+              class A = DEFAULT_ALLOCATOR(T),
               class BC = xtl::xdynamic_bitset<std::size_t>,
               class SA = std::allocator<typename std::vector<T, A>::size_type>>
     using xarray_optional = xarray_container<xtl::xoptional_vector<T, A, BC>, L, DEFAULT_SHAPE_CONTAINER(T, A, SA), xoptional_expression_tag>;
@@ -105,7 +105,10 @@ namespace xt
      * @tparam L The layout_type of the tensor (default: row_major).
      * @tparam A The allocator of the containers holding the elements.
      */
-    template <class T, std::size_t N, layout_type L = DEFAULT_LAYOUT, class A = std::allocator<T>>
+    template <class T,
+              std::size_t N,
+              layout_type L = DEFAULT_LAYOUT,
+              class A = DEFAULT_ALLOCATOR(T)>
     using xtensor = xtensor_container<DEFAULT_DATA_CONTAINER(T, A), N, L>;
 
     template <class EC, std::size_t N, layout_type L = DEFAULT_LAYOUT, class Tag = xtensor_expression_tag>
@@ -124,7 +127,7 @@ namespace xt
     template <class T,
               std::size_t N,
               layout_type L = DEFAULT_LAYOUT,
-              class A = std::allocator<T>,
+              class A = DEFAULT_ALLOCATOR(T),
               class BC = xtl::xdynamic_bitset<std::size_t>>
     using xtensor_optional = xtensor_container<xtl::xoptional_vector<T, A, BC>, N, L, xoptional_expression_tag>;
 
@@ -140,6 +143,24 @@ namespace xt
         {
         };
     }
+
+    namespace evaluation_strategy
+    {
+        struct base
+        {
+        };
+        struct immediate : base
+        {
+        };
+        struct lazy : base
+        {
+        };
+        /*
+        struct cached
+        {
+        };
+        */
+    }
 }
 
 #endif
